﻿using System;
using System.Collections.Generic;
using ClientServer.Models.Exercises.Tests;
using ClientServer.Models.Users;

namespace ClientServer.Helpers
{
    /// <summary>
    /// here are global constants stored that do not change once the app is compiled
    /// these cannot be changed by configs
    ///
    /// e.g. the appsettings.json keys are stored here as strings (all possible keys)
    /// </summary>
    public static class Constants
    {
        public static string VersionString = "2.5.6";

        /// <summary>
        /// this is only set once at program.cs!!
        /// </summary>
        public static string rootDir = "";
        
        /// <summary>
        /// the port to use
        /// </summary>
        public static int Port = 5000;

        /// <summary>
        /// needed when the user downloads personal data and we need to provide the schema some explanation (only the root dir)
        /// </summary>
        public static string SourceCodeUrlBase = "https://gitlab.informatik.uni-halle.de/Syndrom/ClientServer";

        public static string SourceCodeModelsDir = "/tree/master/src/ClientServer/Models";
        
        /// <summary>
        /// break the automatic assessment loop after an assessment threw an error (e.g. test server not reachable...)
        ///
        /// test server not reachable is ok because this gives us an fast error
        /// </summary>
        public static bool BreakAutoAssessmentAfterFirstError = false;


        /// <summary>
        /// the request timeout to ask ostepu for data
        /// </summary>TestAssetForBackend
        public static int OstepuRequestTimeoutInMs = 5000;

        //the internal names MUST exists in the db
        /// <summary>
        /// the internal name for a black box test see <see cref="TestType.InternalName"/>
        /// </summary>
        public const string BlackBoxTestInternalName = "blackBoxTest";
        /// <summary>
        /// the internal name for a compile test see <see cref="TestType.InternalName"/>
        /// </summary>
        public const string CompileTestInternalName = "compileTest";
        /// <summary>
        /// the internal name for a regex test see <see cref="TestType.InternalName"/>
        /// </summary>
        public const string RegexTestInternalName = "regexTest";

        /// <summary>
        /// the internal name for a compare files test see <see cref="TestType.InternalName"/>
        /// </summary>
        public const string CompareFilesTestInternalName = "compareFilesTest";

        
        /// <summary>
        /// the string to split lines in the test protocol
        /// do not change this after we have data in the db!
        /// because all test results are stored as string that we split with this...
        /// a bad design decision... why haven'T we used just \r\n ??
        /// now every new line counts as 4 characters...
        /// </summary>
        public const string TestProtocolSplitString = "\\r\\n";

        /// <summary>
        /// the string used to indicate the arguments in the test protocol
        /// </summary>
        public const string TestProtocolArgsString = "$";
                
        /// <summary>
        /// the prefix for all api paths
        /// MUST BE A CONSt because used in the routes attributes
        /// </summary>
        public const string ApiPrefix = "api/";
      
        public const string InitControllerAbsolutePath = "api/init";
        
        /// <summary>
        /// the auth token cookie name, value is the auth token for the user
        /// </summary>
        public static string AuthTokenCookieName = "auth_token";

        /// <summary>
        /// the cookie send to the frontend
        /// </summary>
        public static string CSRFCookieName = "CSRF-Token";
        
        /// <summary>
        /// requests from the frontend need to include this header and the value should be the token received via
        /// <see cref="CSRFCookieName"/>
        /// </summary>
        public static string CSRFReturnHeaderName = "X-CSRF-Token"; //angular naming

        /// <summary>
        /// this is used in the markdown content (e.g. for an exercise description)
        /// when we include pictures via [img not found](url) we need to specify an url
        /// when we use a normal url and we move servers (e.g. from yapex.informatik... to localhost...)
        /// the url would be invalid so we use an abstraction layer: asset:// is then translated to what we need
        /// e.g. the file could be an inlined blob or we replace it with a real server url
        /// because every context (exercise description / custom project description) knows which assets are connected to them
        /// it is only necessary to specify the asset id and we get the exercise/custom project and can identify the asset uniquely
        /// (not global but with the given context)
        /// if we really want a global url the frontend can transform the url to an absolute url with the given context
        ///
        /// all without changing the markdown content
        /// 
        /// SO wWE use MarkdownAssetFileUrlPrefix + asset id in the markdown e.g. asset://1
        /// this is just here as an reminder, the user in the frontend should use this string when he wants to include an img
        ///   for the backend it has no real meaning (all the transforming is done on the frontend) 
        /// </summary>
        public static string MarkdownAssetFileUrlPrefix = "asset://";

         
        
        //---- appsettings json keys ---
        
       
        public const string IsDebugModeKey = "IsDebugMode";

        public const string IsDbBootstrapEnabledkey = "IsDbBootstrapEnabled";
        
        
        /// <summary>
        /// the key for the system salt in the appsettings
        /// </summary>
        public const string SystemSaltDynamicConfigKey = "SystemSalt";

        /// <summary>
        /// the key for the external login url (ostepu)
        /// </summary>
        public const string ExternalLoginUrl = "ExternalLoginUrl";

        /// <summary>
        /// the main login site
        /// </summary>
        public const string MainEntrySiteKey = "MainEntrySite";

        /// <summary>
        /// the site where the user can set his/her pw for the first the he/she login with ostepu (extern login)
        /// </summary>
        public const string LoginExternalFirstTimeSiteKey = "LoginExternalFirstTimeSite";

 
        /// <summary>
        /// the timeout in minutes for the jwt tokens (in the database)
        /// </summary>
        public const string TokenTimeoutInSConfigKey = "TokenTimeoutInS";


        /// <summary>
        /// the key for the bool indicating if the submit tests should be run async (faster) or sync (safe)
        /// </summary>
        public static string RunAssessmentsAsyncKey = "RunAssessmentsAsync";
        
        /// <summary>
        /// the key for the is slave setting
        /// </summary>
        public static string IsSlaveKey = "IsSlave";


        /// <summary>
        /// the string for the DbConnectionString in the appsettings
        /// </summary>
        public const string DbConnectionKey = "DbConnectionString";

        /// <summary>
        /// the key of the DashboardScriptPathRelativeToCurrentDir setting
        /// </summary>
        public static string DashboardScriptPathRelativeToCurrentDirKey = "DashboardScriptPathRelativeToCurrentDir";

        public static string EnsureFileUploadDirStructureKey = "EnsureFileUploadDirStructure";
        public static string FileUploadDirRelativePathKey = "FileUploadDirRelativePath";
        public static string ExerciseDescriptionFilesUploadDirNameKey = "ExerciseDescriptionFilesUploadDirName";
        public static string TestsFilesUploadDirNameKey = "TestsFilesUploadDirName";
        public static string UserFilesUploadDirNameKey = "UserFilesUploadDirName";

        public static string IsInitControllerEnabledKey = "IsInitControllerEnabled";
        
        public static string OnlySecureFlaggedCookiesKey = "OnlySecureFlaggedCookies";
        
        /// <summary>
        /// used for cors to enable corss site requests, see https://spring.io/understanding/CORS
        /// </summary>
        public static string DebugOriginUrlKey = "DebugOriginUrl";
        /// <summary>
        /// used for cors to enable corss site requests, see https://spring.io/understanding/CORS
        /// </summary>
        public static string DeployOriginUrlsKey = "DeployOriginUrls";


        public static string DeleteFilesWorkerIntervalInMinutesKey = "DeleteFilesWorkerIntervalInMinutes";
        
        public static string MaxUploadFileSizeInByteKey = "MaxUploadFileSizeInByte";
        
        
        
        
        
        
        //--- some core constants ---
        
        /// <summary>
        /// if the program args exceed this length we just drop them
        /// because we cannot just substring in case we used "..." escaped args
        /// frontend has 1000 as length (in description texts)
        /// </summary>
        public static int MaxJustRunProgramArgsLength = 1000;
        
        /// <summary>
        /// the default group display name (when the default group is created via this app)
        /// </summary>
        public static string DefaultGroupDisplayName = "Students";

        /// <summary>
        /// the default group role display name (when the default group role is created via this app)
        /// </summary>
        public static string DefaultGroupRoleDisplayName = "Student Role";

        /// <summary>
        /// the creator group role display name for the creator role (creator of a group) (when the the group role is created via this app)
        /// </summary>
        public static string GroupCreatorRoleDisplayName = "Creator Role";

        /// <summary>
        /// the default system role display name (for the admin role) (when the the system role is created via this app)
        /// </summary>
        public static string DefaultSystemRoleDisplayName = "Admin";

        /// <summary>
        /// the key of the user id for the current session (token sessions)
        /// this is only for the backend because we store the user id in the httpContext session (HttpContext.Items.Add(...))
        /// </summary>
        public static string SessionUserIdKey = "userId";

        /// <summary>
        /// not sure for what this was used...
        /// </summary>
        public static string NeedToExtendTokenLifetimeyKey = "needToExtendTokenLifetimeyKey";


        /// <summary>
        /// we append this to <see cref="SystemSetting.TestServerStatsUrl"/> to get the public stats
        /// </summary>
        public static string TestServerPublicStatsUrlPart = "/public.php";
        /// <summary>
        /// we append this to <see cref="SystemSetting.TestServerStatsUrl"/> to get the private stats
        /// </summary>
        public static string TestServerPrivateStatsUrlPart = "/private.php";

        /// <summary>
        /// probably the db query will fail if we have a X long list via .Contains... so use a magic number here
        /// </summary>
        public static int MaxNumberOfOwnersToQueryInGroup = 1000;

    }
}
