﻿using System.ComponentModel.DataAnnotations;
using ClientServer.Db;
using ClientServer.Models.Exercises.Release;
using ClientServer.Models.Exercises.Tests;
using ClientServer.Models.Interfaces;
using ClientServer.Models.Users;

namespace ClientServer.Models.Exercises.Solution
{
    /// <summary>
    /// stores the result of a run from a <see cref="Test"/>
    /// <para></para>
    /// this is the many-to-many table for <see cref="Test"/> s and <see cref="ExerciseReleaseWithUserAsParticipation"/> s
    /// </summary>
    public class TestWithSingleSolutionAsTestResult : ITestResult
    {

        public int TestId { get; set; }
        /// <summary>
        /// the connected <see cref="Test"/>
        /// </summary>
        public Test Test { get; set; }

        
        public int SolutionPLangId { get; set; }
        public int SolutionUserId { get; set; }
        public int SolutionExerciseReleaseId { get; set; }
        /// <summary>
        /// the connected <see cref="Solution"/>
        /// </summary>
        public Solution Solution { get; set; }


        /// <summary>
        /// true: test for the exercise passed, false: not, null: not ran yet
        /// 
        /// TODO if this is null... this should not exist??
        /// </summary>
        public bool? Passed { get; set; }

        /// <summary>
        /// null: not ran yet, true: code compiled, false: not
        /// </summary>
        public bool? HasCompiled { get; set; }


        /// <summary>
        /// null: not ran yet, the exit code of the user program
        /// </summary>
        public int? ProgramExitCode { get; set; }

        /// <summary>
        /// null: not ran yet, the test result code (see test server codes)
        /// </summary>
        public int? TestResultCode { get; set; }

        /// <summary>
        /// the conversation protocol between the user program and the test runner
        /// </summary>
        [MaxLength(YapexDbContext.TestProtocolMaxStringLength)]
        public string Protocol { get; set; }
        
        /// <summary>
        /// can be null
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string RunnerVersion { get; set; }

        /// <summary>
        /// time for compiling in ms
        /// can be null if compiling was not finished or done
        /// </summary>
        public int? TimeForCompiling { get; set; }
        
        /// <summary>
        /// time needed for execution in ms
        /// can be null if execution was not finished or done
        /// </summary>
        public int? TimeForUserProgram { get; set; }
        
        /// <summary>
        /// null: not ran (e.g. because of test server error)
        /// else: the timeout in ms used for test execution
        /// </summary>
        public int? TimeoutInMsUsed { get; set; }
        
        /// <summary>
        /// null: not ran (e.g. because of test server error)
        /// else: the timeout in ms used for compiling only
        /// </summary>
        public int? CompileTimeoutInMsUsed { get; set; }
        
        /// <summary>
        /// the test server version
        /// can be null
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string TestServerVersion { get; set; }

        /// <summary>
        /// true: the limit <see cref="YapexDbContext.TestProtocolMaxStringLength"/> was exceeded and
        /// the <see cref="Protocol"/> was cut
        /// </summary>
        public bool CharacterLimitExceeded { get; set; }
        /// <summary>
        /// the db will only store a max of characters <see cref="YapexDbContext.TestProtocolMaxStringLength"/> for <see cref="Protocol"/>
        /// can be null for old results
        /// </summary>
        public int? CharacterLimitUsed { get; set; }
    }
}
